import { useState, useEffect, useRef } from "react";
import { VideoJsPlayer } from "video.js";

import { ITraining } from "../../training/types";
import { getExerciseList, getTrainingTime, getPosition } from "../utils";

import Countdown from "./Countdown";
import VideoPlayer from "./VideoPlayer";
import AudioPlayer from "./AudioPlayer";
import { IExerciseItem } from "../types";
import { Howl } from "howler";

const Timer = ({ training }: { training: ITraining }) => {
  const tickPeriod = 100;

  const timeout: {
    current: ReturnType<typeof setTimeout> | undefined;
  } = useRef();

  const [time, setTime] = useState(0);
  const [state, setState] = useState({
    running: false,
    startTime: 0,
    stopTime: 0,
    timeBuffer: 0,
    exerciseList: [] as IExerciseItem[],
    totalTime: 0
  });

  const rosie = useRef(
    new Howl({
      src: ["/media/ROSIE.mp3"],
      sprite: {
        ttt: [114, 2956]
      }
    })
  );

  useEffect(() => {
    //console.log("effect 1");
    const exerciseList = getExerciseList(training.blocks);
    const totalTime = getTrainingTime(exerciseList);
    setState({ ...state, exerciseList, totalTime });
    rosie.current.play("ttt");
  }, [training]);

  const trainingOver = time >= state.totalTime;
  //console.log("is it over?", time, state.totalTime, trainingOver);

  const {
    currentExercise,
    previousExercise,
    nextExercise,
    exerciseTime
  } = getPosition(state.exerciseList, time);

  const video: { current: VideoJsPlayer | undefined } = useRef();
  //const audio: { current: Howl | undefined } = useRef();

  useEffect(() => {
    //console.log("effect 2", time);
    if (!state.running) return;
    timeout.current = setTimeout(tick, tickPeriod - state.timeBuffer);
    setState({ ...state, timeBuffer: 0 });
  }, [time, state.running]);

  function tick() {
    setTime((Date.now() - state.startTime) / 1000);

    if (trainingOver) stopTimer();
  }

  function startTimer() {
    if (trainingOver) return;
    setState({
      ...state,
      running: true,
      startTime: state.startTime + Date.now() - state.stopTime
    });
    if (video.current) video.current.play();
    //if (audio.current) audio.current.play();
    //console.log("Timer started.");
  }

  function stopTimer() {
    if (timeout.current) clearTimeout(timeout.current);

    setState({
      ...state,
      running: false,
      stopTime: Date.now(),
      timeBuffer: Date.now() - state.startTime
    });

    if (video.current) video.current.pause();
    //if (audio.current) audio.current.pause();
    //console.log("stopped");
  }

  function forward() {
    if (nextExercise) {
      setState({
        ...state,
        startTime: state.startTime + 1000 * (time - nextExercise.offset)
      });
      setTime((Date.now() - state.startTime) / 1000);
    }
  }

  function back() {
    if (previousExercise) {
      setState({
        ...state,
        startTime: state.startTime + 1000 * (time - previousExercise.offset)
      });
      setTime((Date.now() - state.startTime) / 1000);
    }
  }

  //console.log("current state:", currentExercise, state);

  return (
    <>
      <div>
        {/*<label htmlFor="rest">Rest</label>
        <input type="number" min="25" max="60" step="5" defaultValue="25" />*/}
        <button onClick={back}>back</button>
        <button
          onClick={state.running ? () => stopTimer() : () => startTimer()}
        >
          {state.running ? "stop" : "start"}
        </button>
        <button onClick={forward}>forward</button>
      </div>
      <div className="content">
        <Countdown
          seconds={
            currentExercise ? currentExercise.duration - exerciseTime : 0
          }
          totalPercent={time / state.totalTime}
          exercisePercent={
            exerciseTime / (currentExercise ? currentExercise.duration : 1)
          }
        />
        <div className="header">current exercise</div>
        <div className="exercise">
          {currentExercise ? currentExercise.exercise : "😎"}
        </div>
        <div className="header">next up</div>
        <div className="exercise">
          {nextExercise ? nextExercise.exercise : "😎"}
        </div>
        <VideoPlayer
          getVideoHandle={(videoHandle: VideoJsPlayer) =>
            (video.current = videoHandle)
          }
        />
      </div>

      <style jsx>{`
        .content {
          text-align: center;
        }

        .header {
          margin-top: 0.4em;
          font-size: 90%;
        }

        .exercise {
          font-size: 110%;
          font-weight: 900;
        }

        :global(.content .video-js) {
          margin: 20px auto;
        }

        @media (min-width: 700px) {
          .content {
            text-align: center;
            display: grid;
            grid-auto-flow: dense;
            grid-template-columns: 1fr auto;
          }
          .header,
          .exercise {
            grid-column: 1/2;
          }
        }
      `}</style>
    </>
  );
};

export default Timer;
